// ==================================================================
//               *** TRACKERBIRD SOFTWARE ANALYTICS ***
//               ******** www.revulytics.com  ********
// ==================================================================
// © 2017 Revulytics, Inc. All rights reserved.
//
// SDK Version: 4.0.7
// Release date: May 2017
//
// You may get the latest version from: http://devzone.revulytics.com
// For customer support please visit: http://helpdesk.revulytics.com
// ==================================================================
//

#import <Foundation/Foundation.h>

#define TBCTrackerbirdOBJC            TBCTrackerbirdOBJC_4_0_7

#define TBC_OK                       1  //Normal execution.  NOTE:  TBC_START_OK (see function startStatus) has different value.
#define TBC_FUNCTION_NOT_AVAIL      -1  //Function not enabled, usage level exceeded, or timeout with the Trackerbird Server.
#define TBC_CONN_ERROR              -2  //General problem connecting with the Trackerbird Server.  See function checkConnectionToURL.
#define TBC_AUTH_FAILURE            -3  //Login denied by the Trackerbird Server.
#define TBC_SERVER_ERROR            -4  //General problem with the Trackerbird Server.
#define TBC_CONFIG_NOT_CREATED      -5  //Function not allowed due to Trackerbird SDK state is prior to successful createConfigWithURL
                                        //that loads Configuration File.
#define TBC_CONFIG_ALREADY_CREATED  -6  //Function not allowed due to Trackerbird SDK state is after successful createConfigWithURL.
#define TBC_NOT_STARTED             -7  //Function not allowed due to Trackerbird SDK state is prior to successful startWithSync or
                                        //after unsuccessful startWithSync.
#define TBC_INVALID_PARAMETER       -9  //Unexpected function parameter value.
#define TBC_INVALID_PATH           -10  //FilePath for Trackerbird Configuration is ill-formed or does not exist.
#define TBC_ACCESS_DENIED          -11  //Permission issues with the Configuration FilePath.
#define TBC_FAILED_DEPENDENCY      -12  //[LINUX only]Problem locating or loading a required dynamic library.
#define TBC_FILE_PATH_NOT_SET      -13  //Function not allowed due to Trackerbird SDK state is prior to setFilePath.
#define TBC_APP_STOPPED            -14  //Function not allowed due to Trackerbird SDK state is after stopWithSync.
#define TBC_INTERNAL_ERROR         -99  //General unrecoverable error.  NOTE:  Once TBC_INTERNAL_ERROR is returned from any
                                        //Trackerbird SDK call, the Client should not make any more Trackerbird SDK calls.
                                        //Any subsequent calls will also return TBC_INTERNAL_ERROR.

#define TBC_START_OK                 2  //[startStatus only]Normal execution.  NOTE:  TBC_OK has different value.
#define TBC_START_LOADING            1  //[startStatus only]Non-error return; Configuration is still loading.
#define TBC_START_NOT_CALLED         0  //[startStatus only]Function not allowed due to Trackerbird SDK state is prior to
                                        //successful startWithSync.
#define TBC_START_CONNECTION_ERROR  -2  //[startStatus only]General problem connecting with the Trackerbird Server.
#define TBC_START_AUTH_FAILURE      -3  //[startStatus only]Login denied by the Trackerbird Server.
#define TBC_START_SERVER_ERROR      -4  //[startStatus only]General problem with the Trackerbird Server.  

#define TBC_CONNCHECK_MISMATCH      -1  //URL and Product ID are mismatched for connection with the Trackerbird Server.

#define TBC_KEYTYPE_UNCHANGED       -1  //Parameter value - Key type is Unchanged (when in param); not used as out param.
#define TBC_KEYTYPE_EVALUATION       0  //Parameter value - Key is of type Evaluation.
#define TBC_KEYTYPE_PURCHASED        1  //Parameter value - Key is of type Purchased.
#define TBC_KEYTYPE_FREEWARE         2  //Parameter value - Key is of type Freeware.
#define TBC_KEYTYPE_UNKNOWN          3  //Parameter value - Key is of type Unknown.
#define TBC_KEYTYPE_NFR              4  //Parameter value - Key is of type Not For Resale.
#define TBC_KEYTYPE_CUSTOM1          5  //Parameter value - Key is of type User Defined - Custom1.
#define TBC_KEYTYPE_CUSTOM2          6  //Parameter value - Key is of type User Defined - Custom2.
#define TBC_KEYTYPE_CUSTOM3          7  //Parameter value - Key is of type User Defined - Custom3.

#define TBC_KEY_STATUS_UNCHANGED    -1  //Parameter value - Key Status Flag is Unchanged (when in param) or Not Known (when out param).
#define TBC_KEY_STATUS_NO            0  //Parameter value - Key Status Flag is No.
#define TBC_KEY_STATUS_YES           1  //Parameter value - Key Status Flag is Yes.

                                        //NOTE:  The Index order below is different than the parameter order in function setLicenseInformationForKeyType.
#define TBC_KEY_ACTIVE_INDEX         0  //Index for License Array - Status Flag for Key being Activated.
#define TBC_KEY_BLACKLIST_INDEX      1  //Index for License Array - Status Flag for Key being Blacklisted.
#define TBC_KEY_EXPIRED_INDEX        2  //Index for License Array - Status Flag for Key being Expired.
#define TBC_KEY_WHITELIST_INDEX      3  //Index for License Array - Status Flag for Key being Whitelisted.
#define TBC_KEY_TYPE_INDEX           4  //Index for License Array - Key Type value.

#define TBC_MESSAGETYPE_ALL          0  //Parameter value - ReachOut™ message type any.
#define TBC_MESSAGETYPE_TEXT         1  //Parameter value - ReachOut™ message type Text only.
#define TBC_MESSAGETYPE_URL          2  //Parameter value - ReachOut™ message type URL only.

#define TBC_START_SYNC               0  //[startWithSync only]Parameter value - Perform Sync and ReachOut™ during start.
#define TBC_START_NO_REACHOUT       -1  //[startWithSync only]Parameter value - Perform Sync but no ReachOut™ during start.
#define TBC_START_NO_SYNC           -2  //[startWithSync only]Parameter value - Perform no Sync nor ReachOut™ during start.

#define TBC_STOP_SYNC                0  //[stopWithSync only]Parameter value - Perform Sync during stop.
#define TBC_STOP_NO_SYNC            -1  //[stopWithSync only]Parameter value - Perform no Sync during stop.

typedef int TBCRESULT;

@interface TBCTrackerbirdOBJC_4_0_7 : NSObject
{
    //Intentionally empty.
}

/**
 * @details This function is the required first call to create a Trackerbird SDK Instance.  The pointer returned is used in all
 * subsequent Trackerbird SDK API calls.  Function init allocates the internal resources for the Trackerbird SDK,
 * including memory, threads, and locks, and on the Linux platform, loads dependent libraries dynamically.  Function
 * init does not create, initialize, nor start the Trackerbird SDK Configuration.  See functions setFilePath
 * (@see setFilePath), createConfigWithURL (@see createConfigWithURL) and startWithSync (@see startWithSync) for more information on initializing
 * a Trackerbird SDK Configuration.
 *
 * IMPORTANT - This object needs to be released (which will call dealloc) when the Trackerbird SDK instance is no longer needed.
 *
 * @exception None.
 *
 * @return One of the following values:
 * * NULL     - Unrecoverable error during Trackerbird SDK creation.
 * * Non-NULL - The pointer to the newly allocated Trackerbird SDK Instance.
 */
-(id)init;

/**
 * @details This function is the required last call to a Trackerbird SDK Instance.  Function dealloc frees up all Trackerbird
 * SDK internal resources, including memory, threads, and locks, and on the Linux platform, closes dependent libraries that
 * were loaded dynamically.
 *
 * IMPORTANT - Do not call function dealloc directly.  It is called automatically by release'ing the Trackerbird SDK object.
 *
 * IMPORTANT - Prior to calling release on the Trackerbird SDK object, the Trackerbird SDK should have been stopped (@see stopWithSync).
 *
 * IMPORTANT - If the Trackerbird SDK has entered TBC_INTERNAL_ERROR state, then dealloc will not actually free up resources.
 *
 * @exception None.
 *
 * @return None.
 */
-(void)dealloc;

/**
 * @details This function returns the SDK Version String for this Trackerbird SDK Instance.  The SDK Version String contains
 * the following formatted information:
 *   Trackerbird SDK version [VER_NUM] for [PLATFORM]/[SHIM]
 *   Where
 *   * VER_NUM      is X.Y.Z (currently 4.0.7)
 *   * PLATFORM     is OSX or LINUX
 *   * SHIM         is ObjC or C
 *
 * IMPORTANT - Call release on this object when it is no longer needed.
 *
 * @exception None.
 *
 * @return One of the following values:
 * * NULL     - The Trackerbird SDK Instance is in a TBC_INTERNAL_ERROR state.
 * * Non-NULL - The pointer to the SDK Version String Object.
 */
-(NSString*)createSDKVersionString;

/**
 * @details This function is used to set the minimum required information to start the Trackerbird SDK Instance.  Function
 * setFilePath (@see setFilePath) must be called before function createConfigWithURL to establish the directory for the Trackerbird
 * SDK Data Files, including the Configuration File.  Function createConfigWithURL should only be called once for an SDK Instance.
 *
 * IMPORTANT - The first time function createConfigWithURL is called, a Configuration File (tbconfig.xml) is created in the
 * designated Data File Path (@see setFilePath).  This Configuration File endures beyond the lifetime of the execution of the
 * Trackerbird SDK Instance, and is consulted on subsequent runs of Trackerbird SDK.  The Configuration File contains most SDK
 * Configuration values including the Callhome URL (@see url).  Once this Configuration File is created, and detected by
 * function createConfigWithURL, the Callhome URL parameter (@see url) supplied to function createConfigWithURL is ignored.
 *
 * IMPORTANT - The Trackerbird Configuration (and hence the Trackerbird SDK Configuration File) may even be shared between
 * multiple SDK Instances, if those SDK Instances use the same Trackerbird SDK Data File Path (@see setFilePath) and same
 * Product ID (@see productID).
 *
 * IMPORTANT - The Callhome URL (@see url) and Product ID (@see productID) are inter-related.  Every Trackerbird Product
 * has a unique URL. Therefore, if during testing one Product ID is used, and then a different Product ID is registered for
 * production, it is important to delete all Trackerbird SDK Data Files.  Otherwise, the Trackerbird SDK Clients will continue
 * to use the testing URL that will now mis-match with the production Product ID.
 *
 * @param url
 * Callhome URL: Every product registered with Trackerbird has its own unique callhome URL usually in the form
 * ‘http://xxxxx.tbnet1.com’. This URL is generated automatically on Account Creation and is used by the Trackerbird SDK to
 * communicate with the Trackerbird server. This URL can be obtained from the Customer Area / Developer Zone.
 *
 * IMPORTANT - url is ignored if a Configuration File already exists, and the URL value within the Configuration File is used
 * instead. This is done so that the Trackerbird Server can instruct the Trackerbird SDK to use a new URL in cases of change.
 * 
 * @param productID
 * This is a unique 10-digit Product ID number that identifies a specific product with the Trackerbird Server. This Product ID
 * is obtained from the Customer Area / Developer Zone.  The combination of Data File Path (@see setFilePath) and Product ID
 * determines whether or not a Configuration File is shared between multiple Trackerbird SDK Instances.
 *
 * @param productVersion
 * The Version Number of the Trackerbird SDK Client (application or plugin) being run.
 *
 * @param productBuildNumber
 * The Build Number of the Trackerbird SDK Client (application or plugin) being run.
 *
 * @param multiSessionEnabled
 * Whether multiple user sessions can be present in a single Trackerbird SDK Instance.  This would typically be false for single
 * user applications, and true for multi-user (e.g., server) applications.  When multiSessionEnabled is false, functions
 * startSession (@see startSession) and stopSession (@see stopSession) are disallowed, and the single session is implied
 * by functions startWithSync (@see startWithSync) and stopWithSync (@see stopWithSync).  When multiSessionEnabled is true, functions startWithSync (@see
 * startWithSync) and stopWithSync (@see stopWithSync) are still required but do not imply a session; i.e., do not record anything.  When
 * multiSessionEnabled is true, functions startSession (@see startSession) and stopSession (@see stopSession) are
 * required to delimit sessions and track events.
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TBC_OK                     (  1) - Normal execution.
 * * TBC_CONFIG_ALREADY_CREATED ( -6) - Function not available; createConfigWithURL has been successfully called before.
 * * TBC_INVALID_PARAMETER      ( -9) - url is NULL or empty, or productID is NULL or empty.
 * * TBC_FAILED_DEPENDENCY      (-12) - [LINUX only] External dynamic library not located or not successfully loaded.
 * * TBC_FILE_PATH_NOT_SET      (-13) - A successful call to setFilePath (@see setFilePath) was not made before this call.
 * * TBC_INTERNAL_ERROR         (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
-(TBCRESULT)createConfigWithURL:(NSString*)url
                      productID:(NSString*)productID
                 productVersion:(NSString*)productVersion
             productBuildNumber:(NSString*)buildNumber
            multiSessionEnabled:(BOOL)multiSessionEnabled;

/**
 * @details This function must be called before utilizing any of the Tracking or Synchronization functions of the Trackerbird
 * SDK.  Before calling function startWithSync, the SDK configuration must be properly set (@see createConfigWithURL).  Function startWithSync
 * should only be called once, and should be paired with a call to function stopWithSync (@see stopWithSync).  Depending on the value of
 * multiSessionEnabled (@see createConfigWithURL), the function startWithSync may start an implied Session.
 *
 * IMPORTANT - Function startWithSync can only be called successfully once.  Calling function startWithSync again even after calling
 * function stopWithSync, will return TBC_FUNCTION_NOT_AVAILABLE.
 *
 * IMPORTANT - Function startWithSync has different return values than function startStatus (@see startStatus).
 *
 * IMPORTANT - Synchronization involves sending cached data stored during this (or previous) Trackerbird SDK execution from the
 * Trackerbird SDK Client to the Trackerbird Server, and receiving Configuration updates for this Trackerbird SDK Instance from
 * the Trackerbird Server, including, potentially, a ReachOut™ message.  The Synchronization process runs in the background
 * during Trackerbird SDK execution.  The Synchronization process is triggered periodically automatically (@see startAutoSyncWithReachout
 * and @see stopAutoSync), and on-demand (@see startWithSync, @see stopWithSync, and @see syncWithReachout).
 * 
 * For more information on ReachOut™ messages, @see checkMessageReturningMessage and @see checkMessageForType.
 *
 * @param doSync
 * Determines if the Trackerbird SDK will Synchronize with the Trackerbird Server immediately at start up.  Normally, doSync is
 * set to TBC_START_SYNC (0).  The possible values are as follows:
 * * TBC_START_SYNC        ( 0) - Start on-demand Synchronization and get ReachOut™ message
 * * TBC_START_NO_REACHOUT (-1) - Start on-demand Synchronization but do not get ReachOut™ message
 * * TBC_START_NO_SYNC     (-2) - Do not start on-demand Synchronization
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TBC_OK                 (  1) - Normal execution.
 * * TBC_FUNCTION_NOT_AVAIL ( -1) - Function ignored; function startWithSync has already been successfully called.
 * * TBC_CONFIG_NOT_CREATED ( -5) - Configuration not successfully loaded.
 * * TBC_INVALID_PARAMETER  ( -9) - doSync is not a legal value.
 * * TBC_INTERNAL_ERROR     (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
-(TBCRESULT)startWithSync:(int)doSync;

/**
 * @details This function must be called after function startWithSync (@see startWithSync), and should always be called when the Trackerbird
 * SDK client is exiting.  Calling function stopWithSync helps the Trackerbird Server to accurately calculate session duration, which
 * is reported in the application usage statistics. Calling function stopWithSync indicates that the SDK Client was terminated
 * gracefully without any unhandled exceptions.  Unlike most Trackerbird SDK functions, function stopWithSync is synchronous, meaning
 * it returns only once finished.  Hence, it is recommended to close/hide any application UI before calling function stopWithSync.
 *
 * IMPORTANT - Synchronization involves sending cached data stored during this (or previous) Trackerbird SDK execution from the
 * Trackerbird SDK Client to the Trackerbird Server, and receiving Configuration updates for this Trackerbird SDK Instance from
 * the Trackerbird Server, including, potentially, a ReachOut™ message.  The Synchronization process runs in the background
 * during Trackerbird SDK execution.  The Synchronization process is triggered periodically automatically (@see startAutoSyncWithReachout
 * and @see stopAutoSync), and on-demand (@see startWithSync, @see stopWithSync, and @see syncWithReachout).
 * 
 * Function stopWithSync does not request ReachOut™ messages during Synchronization.
 *
 * IMPORTANT - Performing a Synchronization will take time.  If the Trackerbird SDK Client needs to exit immediately, setting
 * doSync to TBC_STOP_NO_SYNC (-1) will be the fastest exit.  However, this will defer Synchronization until the next execution
 * of this Trackerbird SDK instance (@see startWithSync, @see syncWithReachout).
 *
 * @param doSync
 * Determines if the Trackerbird SDK will Synchronize with the Trackerbird Server as part of the SDK stop.  Normally, doSync is
 * set to TBC_STOP_SYNC (0).  The possible values are as follows:
 * * TBC_STOP_SYNC      ( 0) - Start on-demand Synchronization and allow it to finish regardless of amount of time taken
 * * TBC_STOP_NO_SYNC   (-1) - Do not start on-demand Synchronization
 * * Any numeric value  >= 1 - Start on-demand Synchronization and allow it to run for a maximum of 'doSync' seconds.  If the
 *       Synchronization is not completed, it is interrupted/aborted, and the data is preserved for a subsequent execution of
 *       Trackerbird SDK.
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TBC_OK                    (  1) - Normal execution.
 * * TBC_NOT_STARTED           ( -7) - Configuration not successfully started.
 * * TBC_INVALID_PARAMETER     ( -9) - doSync is not a legal value.
 * * TBC_APP_STOPPED           (-14) - Function not available as stopWithSync has already been called.
 * * TBC_INTERNAL_ERROR        (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
-(TBCRESULT)stopWithSync:(int)doSync;

/**
 * @details This function will start an on-demand Synchronization.  In general, the periodic automatic Synchronization is
 * sufficient, and there is no need to call function syncWithReachout.  Function syncWithReachout returns immediately, with the Synchronization
 * process running asynchronously in the background.  Hence, there is no need to place a call to function syncWithReachout inside a
 * separate Client thread.
 *
 * IMPORTANT - Synchronization involves sending cached data stored during this (or previous) Trackerbird SDK execution from the
 * Trackerbird SDK Client to the Trackerbird Server, and receiving Configuration updates for this Trackerbird SDK Instance from
 * the Trackerbird Server, including, potentially, a ReachOut™ message.  The Synchronization process runs in the background
 * during Trackerbird SDK execution.  The Synchronization process is triggered periodically automatically (@see startAutoSyncWithReachout
 * and @see stopAutoSync), and on-demand (@see startWithSync, @see stopWithSync, and @see syncWithReachout).
 *
 * For more information on ReachOut™ messages, @see checkMessageReturningMessage and @see checkMessageForType.
 *
 * Both the Trackerbird SDK and the Trackerbird Server can reject an on-demand Synchronization request from occurring.  This is
 * done to prevent abuse and unnecessary server load.
 *
 * @param getReachout
 * Determines if the Trackerbird Server will send a ReachOut™ message during this particular Synchronization, if available.
 * Normally, the value should be true.
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TBC_OK                    (  1) - Normal execution.
 * * TBC_FUNCTION_NOT_AVAIL    ( -1) - Function not available due to empty Client ID.
 * * TBC_NOT_STARTED           ( -7) - Configuration not successfully started.
 * * TBC_APP_STOPPED           (-14) - Function not available as stopWithSync has already been called.
 * * TBC_INTERNAL_ERROR        (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
-(TBCRESULT)syncWithReachout:(BOOL)getReachout;

/**
 * @details This function will start the automatic periodic Synchronization in the background.  This is usually only required
 * for Clients that cannot call function stopWithSync for Synchronization, such as background services or web services whose running
 * cycle (session runtime duration) spans over 6 hours.  Before using this function please make sure you read the section on
 * Caching and Synchronizing in the on-line documentation.  The automatic periodic Synchronization can be stopped with function
 * stopAutoSync (@see stopAutoSync).  After stopping the automatic Synchronization, it can be re-started with another call
 * to function startAutoSyncWithReachout.
 *
 * IMPORTANT - Synchronization involves sending cached data stored during this (or previous) Trackerbird SDK execution from the
 * Trackerbird SDK Client to the Trackerbird Server, and receiving Configuration updates for this Trackerbird SDK Instance from
 * the Trackerbird Server, including, potentially, a ReachOut™ message.  The Synchronization process runs in the background
 * during Trackerbird SDK execution.  The Synchronization process is triggered periodically automatically (@see startAutoSyncWithReachout
 * and @see stopAutoSync), and on-demand (@see startWithSync, @see stopWithSync, and @see syncWithReachout).
 *
 * For more information on ReachOut™ messages, @see checkMessageReturningMessage and @see checkMessageForType.
 *
 * @param getReachout
 * Determines if the Trackerbird Server will send a ReachOut™ message during automatic Synchronization, if available.  Normally,
 * the value should be true.
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TBC_OK                    (  1) - Normal execution.
 * * TBC_NOT_STARTED           ( -7) - Configuration not successfully started.
 * * TBC_APP_STOPPED           (-14) - Function not available as stopWithSync has already been called.
 * * TBC_INTERNAL_ERROR        (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
-(TBCRESULT)startAutoSyncWithReachout:(BOOL)getReachout;

/**
 * @details This function will stop the automatic periodic Synchronization in the background.  For more information see function
 * startAutoSyncWithReachout (@see startAutoSyncWithReachout).
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TBC_OK                    (  1) - Normal execution.
 * * TBC_NOT_STARTED           ( -7) - Configuration not successfully started.
 * * TBC_APP_STOPPED           (-14) - Function not available as stopWithSync has already been called.
 * * TBC_INTERNAL_ERROR        (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
-(TBCRESULT)stopAutoSync;

/**
 * @details This function starts a new User Session with the supplied unique Session ID.  Functions startSession and
 * stopSession (@see stopSession) should be called only if multiSessionEnabled was set to true in function createConfigWithURL
 * (@see createConfigWithURL).  Function startSession records the time of the call to use in the Trackerbird Session Duration
 * calculation.  More than one User Session can exist simultaneously, each with an unique Session ID.  Trackerbird Event
 * Tracking can be associated to a given User Session (@see trackEventWithCategory, @see trackEventWithCategory:CustomNum, and @see trackEventWithCategory:CustomText).
 *
 * @param sessionID
 * An unique ID that refers to the user session that is being started.  This same Session ID should later be used for Event
 * Tracking.  The value must not be NULL, must not be empty, and must be unique.
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TBC_OK                    (  1) - Normal execution.
 * * TBC_FUNCTION_NOT_AVAIL    ( -1) - createConfigWithURL was not called with multiSessionEnabled = true.
 * * TBC_NOT_STARTED           ( -7) - Configuration not successfully started.
 * * TBC_INVALID_PARAMETER     ( -9) - sessionID is NULL, empty, or not unique (in use).
 * * TBC_APP_STOPPED           (-14) - Function not available as stopWithSync has already been called.
 * * TBC_INTERNAL_ERROR        (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
-(TBCRESULT)startSession:(NSString*)sessionID;

/**
 * @details This function ends the User Session for the supplied Session ID.  Functions startSession (@see startSession) and
 * stopSession should be called only if multiSessionEnabled was set to true in function createConfigWithURL (@see createConfigWithURL).
 * Function stopSession records the time of the call to use in the Trackerbird Session Duration calculation.  If function
 * stopSession is never called for a given Session ID, eventually the Trackerbird SDK will consider the User Session as "timed
 * out", and the last recorded Event (@see trackEventWithCategory, @see trackEventWithCategory:CustomNum, and @see trackEventWithCategory:CustomText) for the User Session
 * will be used as the User Session Stop Time.
 *
 * @param sessionID
 * A Session ID previously used in a call to function startSession (@see startSession) that has not yet been used in a call
 * to function stopSession.
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TBC_OK                    (  1) - Normal execution.
 * * TBC_FUNCTION_NOT_AVAIL    ( -1) - createConfigWithURL was not called with multiSessionEnabled = true.
 * * TBC_NOT_STARTED           ( -7) - Configuration not successfully started.
 * * TBC_INVALID_PARAMETER     ( -9) - sessionID not recognized (NULL, empty, not used in startSession, already used in stopSession).
 * * TBC_APP_STOPPED           (-14) - Function not available as stopWithSync has already been called.
 * * TBC_INTERNAL_ERROR        (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
-(TBCRESULT)stopSession:(NSString*)sessionID;

/**
 * @details This function sets the Trackerbird SDK Data File Path that will be used to locate, create and save its data files.
 * Function setFilePath must be called before calling function createConfigWithURL (@see createConfigWithURL), and cannot be called
 * after calling function createConfigWithURL.
 *
 * IMPORTANT - The combination of Trackerbird SDK Data File Path and Product ID (@see createConfigWithURL) determine whether or not
 * the Trackerbird Configuration is shared between multiple Trackerbird SDK Instances.
 *
 * IMPORTANT - The Trackerbird SDK Client application must have read/write access to the Data File Path.
 *
 * @param filePath
 * The location to use for the Trackerbird SDK Data Files.  The value must not be NULL nor empty.  The value must refer to a
 * directory that exists and has write permissions for this Trackerbird Client process.
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TBC_OK                     (  1) - Normal execution.
 * * TBC_CONFIG_ALREADY_CREATED ( -6) - Function ignored as filePath cannot be changed after call to createConfigWithURL.
 * * TBC_INVALID_PARAMETER      ( -9) - filePath is NULL or empty.
 * * TBC_INVALID_PATH           (-10) - filePath does not exist.
 * * TBC_ACCESS_DENIED          (-11) - filePath does not have write permissions.
 * * TBC_INTERNAL_ERROR         (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
-(TBCRESULT)setFilePath:(NSString*)filePath;

/**
 * @details This function sets the Product Edition to the supplied value.  An example of this would be to distinguish between
 * "Home" and "Business" modes within a single Product ID (@see createConfigWithURL).  Function setProductEdition does not have
 * to be called, but if it is called, it must be after calling function createConfigWithURL (@see createConfigWithURL).  Function
 * setProductEdition can be called more than once.
 *
 * IMPORTANT - Function setProductEdition does not have to be called, but if it is called, it must be after calling function
 * createConfigWithURL (@see createConfigWithURL), and ideally before function startWithSync (@see startWithSync), especially if this is a new
 * installation.  For a new installation, the value set will be communicated to the server immediately if set prior to calling
 * function startWithSync, or on the next sync if set after calling function startWithSync.
 *
 * @param productEdition
 * The Product Edition value to use.  A NULL or empty value removes any previous value.
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TBC_OK                 (  1) - Normal execution.
 * * TBC_CONFIG_NOT_CREATED ( -5) - Configuration not successfully loaded.
 * * TBC_INTERNAL_ERROR     (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
-(TBCRESULT)setProductEdition:(NSString*)productEdition;

/**
 * @details This function sets the Product Language to the supplied value.  An example of this would be to distinguish the
 * language a user views an internationalized product in.  Note, Product Language is different than the OS Language that is
 * automatically collected by the Trackerbird SDK.
 *
 * IMPORTANT - Function setProductLanguage does not have to be called, but if it is called, it must be after calling function
 * createConfigWithURL (@see createConfigWithURL), and ideally before function startWithSync (@see startWithSync), especially if this is a new
 * installation.  For a new installation, the value set will be communicated to the server immediately if set prior to calling
 * function startWithSync, or on the next sync if set after calling function startWithSync.
 *
 * @param productLanguage
 * The Product Language value to use.  A NULL or empty value removes any previous value.
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TBC_OK                 (  1) - Normal execution.
 * * TBC_CONFIG_NOT_CREATED ( -5) - Configuration not successfully loaded.
 * * TBC_INTERNAL_ERROR     (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
-(TBCRESULT)setProductLanguage:(NSString*)productLanguage;

/**
 * @details This function sets the Product Version to the supplied value.  In general, function setProductVersion does not
 * need to be called, as the Product Version is set during createConfigWithURL (@see createConfigWithURL).  Function setProductVersion
 * is used to change the Product Version from its original value, for example, in the case of an on-line Client update.
 *
 * IMPORTANT - Function setProductVersion does not have to be called, but if it is called, it must be after calling function
 * createConfigWithURL (@see createConfigWithURL), and ideally before function startWithSync (@see startWithSync), especially if this is a new
 * installation.  For a new installation, the value set will be communicated to the server immediately if set prior to calling
 * function startWithSync, or on the next sync if set after calling function startWithSync.
 *
 * @param productVersion
 * The Product Version value to use.  A NULL or empty value removes any previous value.
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TBC_OK                 (  1) - Normal execution.
 * * TBC_CONFIG_NOT_CREATED ( -5) - Configuration not successfully loaded.
 * * TBC_INTERNAL_ERROR     (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
-(TBCRESULT)setProductVersion:(NSString*)productVersion;

/**
 * @details This function sets the Product Build Number to the supplied value.  In general, function setProductBuildNumber
 * does not need to be called, as the Product Build Number is set during createConfigWithURL (@see createConfigWithURL).  Function
 * setProductBuildNumber is used to change the Product Build Number from its original value, for example, in the case of an
 * on-line Client update.
 *
 * IMPORTANT - Function setProductBuildNumber does not have to be called, but if it is called, it must be after calling function
 * createConfigWithURL (@see createConfigWithURL), and ideally before function startWithSync (@see startWithSync), especially if this is a new
 * installation.  For a new installation, the value set will be communicated to the server immediately if set prior to calling
 * function startWithSync, or on the next sync if set after calling function startWithSync.
 *
 * @param productBuildNumber
 * The Product Build Number value to use.  A NULL or empty value removes any previous value.
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TBC_OK                 (  1) - Normal execution.
 * * TBC_CONFIG_NOT_CREATED ( -5) - Configuration not successfully loaded.
 * * TBC_INTERNAL_ERROR     (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
-(TBCRESULT)setProductBuildNumber:(NSString*)productBuildNumber;

/**
 * @details This function sets the License Information being used by the Trackerbird SDK Client.  The License Information
 * includes a License Key Type and four (4) License Status Flags -- keyExpired, keyActivated, keyBlacklisted, and
 * keyWhitelisted.  The License Key Type can have one of eight (8) values listed below, including any of three (3) custom values
 * that may be used freely by the Client.  If a License Status Flag value is not to be changed, the value
 * TBC_KEY_STATUS_UNCHANGED (-1) can be used for the corresponding parameter.  The License Key Type can also be indicated as
 * unchanged with value TBC_KEYTYPE_UNCHANGED (-1).
 *
 * IMPORTANT - Function setLicenseInformationForKeyType does not have to be called, but if it is called, it must be after calling function
 * createConfigWithURL (@see createConfigWithURL), and ideally before function startWithSync (@see startWithSync), especially if this is a new
 * installation.  For a new installation, the value set will be communicated to the server immediately if set prior to calling
 * function startWithSync, or on the next sync if set after calling function startWithSync.
 *
 * IMPORTANT - For a new installation, not calling function setLicenseInformationForKeyType before function startWithSync (@see startWithSync) may result in
 * anomolies in the License Funnel Report, as the Key Type will initially be recorded as "Unknown" and later recorded as the set
 * value.
 *
 * IMPORTANT - The parameter order in function setLicenseInformationForKeyType is not the same as the index order in the License Array (see Index
 * #defines TBC_KEY_[NAME]_INDEX.)
 *
 * @param keyType
 * The Key Type to set.  The possible values are as follows:
 * * TBC_KEYTYPE_UNCHANGED  (-1)
 * * TBC_KEYTYPE_EVALUATION ( 0)
 * * TBC_KEYTYPE_PURCHASED  ( 1)
 * * TBC_KEYTYPE_FREEWARE   ( 2)
 * * TBC_KEYTYPE_UNKNOWN    ( 3)
 * * TBC_KEYTYPE_NFR        ( 4)
 * * TBC_KEYTYPE_CUSTOM1    ( 5)
 * * TBC_KEYTYPE_CUSTOM2    ( 6)
 * * TBC_KEYTYPE_CUSTOM3    ( 7)
 *
 * @param keyExpired
 * The License Status Flag to indicate whether the Client license has expired.  The possible values are as follows:
 * * TBC_KEY_STATUS_UNCHANGED (-1)
 * * TBC_KEY_STATUS_NO        ( 0)
 * * TBC_KEY_STATUS_YES       ( 1)
 *
 * @param keyActivated
 * The License Status Flag to indicate whether the Client license been activated.  The possible values are as follows:
 * * TBC_KEY_STATUS_UNCHANGED (-1)
 * * TBC_KEY_STATUS_NO        ( 0)
 * * TBC_KEY_STATUS_YES       ( 1)
 *
 * @param keyBlacklisted
 * The License Status Flag to indicate whether the Client license has been blacklisted.  The possible values are as follows:
 * * TBC_KEY_STATUS_UNCHANGED (-1)
 * * TBC_KEY_STATUS_NO        ( 0)
 * * TBC_KEY_STATUS_YES       ( 1)
 *
 * @param keyWhitelisted
 * The License Status Flag to indicate whether the Client license has been whitelisted.  The possible values are as follows:
 * * TBC_KEY_STATUS_UNCHANGED (-1)
 * * TBC_KEY_STATUS_NO        ( 0)
 * * TBC_KEY_STATUS_YES       ( 1)
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TBC_OK                 (  1) - Normal execution.
 * * TBC_CONFIG_NOT_CREATED ( -5) - Configuration not successfully loaded.
 * * TBC_INVALID_PARAMETER  ( -9) - keyType, keyExpired, keyActivated, keyBlacklisted or keyWhitelisted is not a legal value.
 * * TBC_INTERNAL_ERROR     (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
-(TBCRESULT)setLicenseInformationForKeyType:(int)keyType
                              expiredStatus:(int)expiredStatus
                            activatedStatus:(int)activatedStatus
                          blacklistedStatus:(int)blacklistedStatus
                          whitelistedStatus:(int)whitelistedStatus;

/**
 * @details This function sets one of the twenty (20) Custom Properties to the supplied value.  On the Trackerbird Web UI,
 * Custom Values are given an ID ranging from C01 to C20, corresponding to the supplied index to function setCustomProperty.
 *
 * IMPORTANT - Function setCustomProperty does not have to be called, but if it is called, it must be after calling function
 * createConfigWithURL (@see createConfigWithURL).
 *
 * @param index
 * A value between 1 and 20.  NOTE, this is not a zero-based index.
 *
 * @param customValue
 * The value to be assigned to the indexed Custom Property.  A NULL or empty value removes any previous value. The maximum
 * length of customValue is 1024 characters not including the terminating NULL.
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TBC_OK                 (  1) - Normal execution.
 * * TBC_CONFIG_NOT_CREATED ( -5) - Configuration not successfully loaded.
 * * TBC_INVALID_PARAMETER  ( -9) - Index out of range (1-20); customValue max length exceeded.
 * * TBC_INTERNAL_ERROR     (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
-(TBCRESULT)setCustomProperty:(NSString*)property
                      atIndex:(unsigned short)index;

/**
 * @details This function returns the current status of function startWithSync (@see startWithSync).  NOTE, function startWithSync (@see startWithSync)
 * executes asynchronously, so the status returned by startStatus may change over time.
 *
 * IMPORTANT - Function startStatus does not have to be called, but if it is called, it must be after calling function
 * startWithSync (@see startWithSync).
 *
 * IMPORTANT - The return codes for function startStatus are different than those for function startWithSync (@see startWithSync).
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TBC_START_OK               (  2) - Normal execution, startWithSync has completed successfully.
 * * TBC_START_LOADING          (  1) - Normal execution, startWithSync is still working.
 * * TBC_START_NOT_CALLED       (  0) - startWithSync has not been called.
 * * TBC_START_CONNECTION_ERROR ( -2) - startWithSync has completed unsuccessfully due to comms issue with the Trackerbird Server.
 * * TBC_START_AUTH_FAILURE     ( -3) - startWithSync has completed unsuccessfully due to login issue with the Trackerbird Server.
 * * TBC_START_SERVER_ERROR     ( -4) - startWithSync has completed unsuccessfully due to internal issue with the Trackerbird Server.
 * * TBC_INTERNAL_ERROR         (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
-(TBCRESULT)startStatus;

/**
 * @details This function is used to Track and Report an Exception that has been generated by the Trackerbird SDK Client.
 *
 * IMPORTANT - Function trackExceptionWithClassName does not have to be called, but if it is called, it must be after calling function
 * startWithSync (@see startWithSync).
 *
 * @param className
 * The name of the class from which the Exception occurred.  The value can be NULL or empty, but cannot contain a NEWLINE char.
 *
 * @param methodName
 * The name of the method/function from which the Exception occurred.  The value can be NULL or empty, but cannot contain a
 * NEWLINE char.
 *
 * @param exceptionMessage
 * The message contained in the Exception or text to be logged with the Exception.  The value can be NULL or empty.  The value
 * can contain multiple lines.
 *
 * @param stackTrace
 * The full or partial stack trace of the Exception to be logged with the Exception.  The value can be NULL or empty.  The value
 * can contain multiple lines.
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TBC_OK                    (  1) - Normal execution.
 * * TBC_FUNCTION_NOT_AVAIL    ( -1) - Exception tracking feature is disabled.
 * * TBC_NOT_STARTED           ( -7) - Configuration not successfully started.
 * * TBC_INVALID_PARAMETER     ( -9) - className or methodName contains newline character.
 * * TBC_APP_STOPPED           (-14) - Function not available as stopWithSync has already been called.
 * * TBC_INTERNAL_ERROR        (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
-(TBCRESULT)trackExceptionWithClassName:(NSString*)className
                             methodName:(NSString*)methodName
                       exceptionMessage:(NSString*)message
                             stackTrace:(NSString*)stackTrace;

/**
 * @details This function is used to Track an Event in the Trackerbird SDK Client which no Custom Value needs to be recorded.  To
 * record a Custom Value with the Event use trackEventWithCategory:CustomNum (@see trackEventWithCategory:CustomNum) or trackEventWithCategory:CustomText (@see trackEventWithCategory:CustomText).
 * 
 * An Event must have a non-empty Event Name.  If User Sessions are being used (@see createConfigWithURL and @see startSession)
 * an Event must have a recognized Session ID.  If User Sessions are not being used (@see createConfigWithURL), an Event cannot have
 * a Session ID (it must be NULL or empty).
 *
 * Optionally, Categories can be used to organize Events on the Trackerbird Reports, making the Event Data more readable.  Each
 * Event can be associated with a single Category via the supplied parameter.  If a Category Name is not desired, this parameter
 * should be NULL or empty.
 *
 * Event Names and Categories have restricted lengths.  Exceeding the maximum length results in truncation of the value.  This is
 * influenced by allowExtendedNames:
 * * If allowExtendedNames is true, Event Name and Category can each be up to 256 characters, not including the terminating
 *       NULL, with no further restriction on their combined length.
 * * If allowExtendedNames is false, Event Name and Category can each be up to 40 characters (not including the terminating
 *       NULL), and combined, they can be up to 50 characters (not including the terminating NULL).  Exceeding the combined
 *       length restriction preserves the Category (up to 40 characters), with the remaining length used by as much of the
 *       Event Name that can fit.
 *
 * IMPORANT - The Event Name and Category will be visible in the Trackerbird Analytics Reports.  Therefore, meaningful and
 * structured names are recommended to make it easier to identify Event Data in the Reports.
 *
 * @param category
 * The name of the Category that this Event belongs. A value of NULL or empty means no Category.  The value cannot contain a
 * NEWLINE character.  The value is length constrained and is influenced by allowExtendedNames and potentially influenced by the
 * length of eventName.
 *
 * @param eventName
 * The name of the Event to Track. The value cannot be NULL, empty nor contain a NEWLINE character.  The value is length
 * constrained and is influenced by allowExtendedNames and potentially influenced by the length of category.
 *
 * @param sessionID
 * The legal values depend on whether multiple Sessions are enabled (@see createConfigWithURL).
 * * If multiple Sessions are enabled, then the value must be a Session ID previously used in a call to function startSession
 *       (@see startSession) that has not yet been used in a call to function stopSession (@see stopSession).
 * * If multiple Sessions are not enabled, then the value must be NULL or empty.
 *
 * @param allowExtendedNames
 * Influences the individual and combined lengths of Event Name and Event Category.  See details above.
 * 
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TBC_OK                    (  1) - Normal execution.  NOTE:  If the given event has not been whitelisted in the Trackerbird
 *                                    server, then the event will not actually be tracked.
 * * TBC_FUNCTION_NOT_AVAIL    ( -1) - event tracking feature can be disabled.
 * * TBC_NOT_STARTED           ( -7) - Configuration not successfully started.
 * * TBC_INVALID_PARAMETER     ( -9) - eventName NULL, empty or contains newline character; category has
 *                                    newline char; sessionID not recognized and multiSessionEnabled = true; or sessionID
 *                                    not NULL/empty and multiSessionEnabled = false.
 * * TBC_APP_STOPPED           (-14) - Function not available as stopWithSync has already been called.
 * * TBC_INTERNAL_ERROR        (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
-(TBCRESULT)trackEventWithCategory:(NSString*)category
                         eventName:(NSString*)eventName
                         sessionID:(NSString*)sessionID
                allowExtendedNames:(BOOL)allowExtendedNames;

/**
 * @details This function is used to Track an Event in the Trackerbird SDK Client which needs to record a numeric Custom Value.
 * To record a textual Custom Value with the Event use trackEventWithCategory:CustomText (@see trackEventWithCategory:CustomText), and use trackEventWithCategory (@see
 * trackEventWithCategory) when no Custom Value is needed.  The supplied numeric Custom Value is used in the Trackerbird Event Tracking
 * Report to produce averages.  This is typically used to record the time it takes for a process to finish, or some other
 * numeric value that can be used to calculate a specific trend.
 * 
 * An Event must have a non-empty Event Name.  If User Sessions are being used (@see createConfigWithURL and @see startSession)
 * an Event must have a recognized Session ID.  If User Sessions are not being used (@see createConfigWithURL), an Event cannot have
 * a Session ID (it must be NULL or empty).
 *
 * Optionally, Categories can be used to organize Events on the Trackerbird Reports, making the Event Data more readable.  Each
 * Event can be associated with a single Category via the supplied parameter.  If a Category Name is not desired, this parameter
 * should be NULL or empty.
 *
 * Event Names and Categories have restricted lengths.  Exceeding the max length results in truncation of the value.  This is
 * influenced by allowExtendedNames:
 * * If allowExtendedNames is true, Event Name and Category can each be up to 256 characters, not including the terminating
 *       NULL, with no further restriction on their combined length.
 * * If allowExtendedNames is false, Event Name and Category can each be up to 40 characters (not including the terminating
 *       NULL), and combined, they can be up to 50 characters (not including the terminating NULL).  Exceeding the combined
 *       length restriction preserves the Category (up to 40 characters), with the remaining length used by as much of the
 *       Event Name that can fit.
 *
 * IMPORANT - The Event Name and Category will be visible in the Trackerbird Analytics Reports.  Therefore, meaningful and
 * structured names are recommended to make it easier to identify Event Data in the Reports.
 *
 * @param category
 * The name of the Category that this Event belongs. A value of NULL or empty means no Category.  The value cannot contain a
 * NEWLINE character.  The value is length constrained and is influenced by allowExtendedNames and potentially influenced by the
 * length of eventName.
 *
 * @param eventName
 * The name of the Event to Track. The value cannot be NULL, empty nor contain a NEWLINE character.  The value is length
 * constrained and is influenced by allowExtendedNames and potentially influenced by the length of category.
 *
 * @param customValue
 * A numeric Custom Value related to this particular event.
 *
 * @param sessionID
 * The legal values depend on whether multiple Sessions are enabled (@see createConfigWithURL).
 * * If multiple Sessions are enabled, then the value must be a Session ID previously used in a call to function startSession
 *       (@see startSession) that has not yet been used in a call to function stopSession (@see stopSession).
 * * If multiple Sessions are not enabled, then the value must be NULL or empty.
 *
 * @param allowExtendedNames
 * Influences the individual and combined lengths of Event Name and Event Category.  See details above.
 * 
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TBC_OK                    (  1) - Normal execution.  NOTE:  If the given event has not been whitelisted in the Trackerbird
 *                                    server, then the event will not actually be tracked.
 * * TBC_FUNCTION_NOT_AVAIL    ( -1) - Exception tracking feature is disabled.
 * * TBC_NOT_STARTED           ( -7) - Configuration not successfully started.
 * * TBC_INVALID_PARAMETER     ( -9) - eventName NULL, empty or contains newline character; category has
 *                                    newline char; sessionID not recognized and multiSessionEnabled = true; or sessionID
 *                                    not NULL/empty and multiSessionEnabled = false.
 * * TBC_APP_STOPPED           (-14) - Function not available as stopWithSync has already been called.
 * * TBC_INTERNAL_ERROR        (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
-(TBCRESULT)trackEventWithCategory:(NSString*)category
                         eventName:(NSString*)eventName
                         customNum:(NSNumber*)value
                         sessionID:(NSString*)sessionID
                allowExtendedNames:(BOOL)allowExtendedNames;

/**
 * @details This function is used to Track an Event in the Trackerbird SDK Client which needs to record a textual Custom Value.
 * To record a numeric Custom Value with the Event use trackEventWithCategory:CustomNum (@see trackEventWithCategory:CustomNum), and use trackEventWithCategory (@see
 * trackEventWithCategory) when no Custom Value is needed.  The supplied textual Custom Value is used in the Trackerbird Event Tracking
 * Report.  This is typically used to record data collected from the Client or User, such as comments, the brand of GPU on the
 * machine, etc.
 * 
 * An Event must have a non-empty Event Name.  If User Sessions are being used (@see createConfigWithURL and @see startSession)
 * an Event must have a recognized Session ID.  If User Sessions are not being used (@see createConfigWithURL), an Event cannot have
 * a Session ID (it must be NULL or empty).
 *
 * Optionally, Categories can be used to organize Events on the Trackerbird Reports, making the Event Data more readable.  Each
 * Event can be associated with a single Category via the supplied parameter.  If a Category Name is not desired, this parameter
 * should be NULL or empty.
 *
 * Event Names and Categories have restricted lengths.  Exceeding the max length results in truncation of the value.  This is
 * influenced by allowExtendedNames:
 * * If allowExtendedNames is true, Event Name and Category can each be up to 256 characters, not including the terminating
 *       NULL, with no further restriction on their combined length.
 * * If allowExtendedNames is false, Event Name and Category can each be up to 40 characters (not including the terminating
 *       NULL), and combined, they can be up to 50 characters (not including the terminating NULL).  Exceeding the combined
 *       length restriction preserves the Category (up to 40 characters), with the remaining length used by as much of the
 *       Event Name that can fit.
 *
 * IMPORANT - The Event Name and Category will be visible in the Trackerbird Analytics Reports.  Therefore, meaningful and
 * structured names are recommended to make it easier to identify Event Data in the Reports.
 *
 * @param category
 * The name of the Category that this Event belongs. A value of NULL or empty means no Category.  The value cannot contain a
 * NEWLINE character.  The value is length constrained and is influenced by allowExtendedNames and potentially influenced by the
 * length of eventName.
 *
 * @param eventName
 * The name of the Event to Track. The value cannot be NULL, empty nor contain a NEWLINE character.  The value is length
 * constrained and is influenced by allowExtendedNames and potentially influenced by the length of category.
 *
 * @param customValue
 * A textual Custom Value related to this particular event. The max length of this text is 4096 characters, but this can be
 * changed via a request to Trackerbird.
 *
 * @param sessionID
 * The legal values depend on whether multiple Sessions are enabled (@see createConfigWithURL).
 * * If multiple Sessions are enabled, then the value must be a Session ID previously used in a call to function startSession
 *       (@see startSession) that has not yet been used in a call to function stopSession (@see stopSession).
 * * If multiple Sessions are not enabled, then the value must be NULL or empty.
 *
 * @param allowExtendedNames
 * Influences the individual and combined lengths of Event Name and Event Category.  See details above.
 * 
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TBC_OK                    (  1) - Normal execution.  NOTE:  If the given event has not been whitelisted in the Trackerbird
 *                                    server, then the event will not actually be tracked.
 * * TBC_FUNCTION_NOT_AVAIL    ( -1) - Exception tracking feature is disabled.
 * * TBC_NOT_STARTED           ( -7) - Configuration not successfully started.
 * * TBC_INVALID_PARAMETER     ( -9) - eventName NULL, empty or contains newline character; category has
 *                                    newline char; sessionID not recognized and multiSessionEnabled = true; or sessionID
 *                                    not NULL/empty and multiSessionEnabled = false.
 * * TBC_APP_STOPPED           (-14) - Function not available as stopWithSync has already been called.
 * * TBC_INTERNAL_ERROR        (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
-(TBCRESULT)trackEventWithCategory:(NSString*)category
                         eventName:(NSString*)eventName
                        customText:(NSString*)value
                         sessionID:(NSString*)sessionID
                allowExtendedNames:(BOOL)allowExtendedNames;

/**
 * @details This function tests the connectivity between the Trackerbird SDK and the Trackerbird Server.  Function checkConfig
 * is similar to function checkConnectionToURL (@see checkConnectionToURL) but uses the Trackerbird SDK Configuration Data (@see
 * createConfigWithURL) for the CallHome URL and Product ID values instead of function parameters.
 *
 * IMPORTANT - Function checkConfig is generally not needed, as each Trackerbird SDK API requiring connectivity to the
 * Trackerbird Server does its own connectivity test.  If function checkConfig is called, it must be after calling function
 * createConfigWithURL (@see createConfigWithURL).  Function checkConnectionToURL (@see checkConnectionToURL) can be called before calling
 * function createConfigWithURL.
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TBC_OK                 (  1) - Normal execution.
 * * TBC_CONNCHECK_MISMATCH ( -1) - Successfully reached the Trackerbird Server, but the URL and ProductID are not consistent.
 * * TBC_CONN_ERROR         ( -2) - Unsuccessful due to communications issue with the Trackerbird Server.
 * * TBC_AUTH_FAILURE       ( -3) - Unsuccessful due to login issue with the Trackerbird Server.
 * * TBC_SERVER_ERROR       ( -4) - Unsuccessful due to internal issue with the Trackerbird Server.
 * * TBC_CONFIG_NOT_CREATED ( -5) - Configuration not successfully loaded.
 * * TBC_INTERNAL_ERROR     (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
-(TBCRESULT)checkConfig;

/**
 * @details This function tests the connectivity between the Trackerbird SDK and the Trackerbird Server.  Function
 * checkConnectionToURL is similar to function checkConfig (@see checkConfig) but uses parameters for the CallHome URL and
 * Product ID rather than the Trackerbird SDK Configuration Data.
 * 
 * IMPORTANT - Function checkConnectionToURL is generally not needed, as each Trackerbird SDK API requiring connectivity to the
 * Trackerbird Server does its own connectivity test.  Function checkConnectionToURL can be called before calling function
 * createConfigWithURL (@see createConfigWithURL).
 *
 * @param url
 * The URL to test connectivity to (usually the Trackerbird CallHome URL).
 *
 * @param productID
 * Optionally, the Product ID for the Trackerbird Client.
 * * If NULL or empty, the Product ID is not used.
 * * Otherwise, the connectivity tests will validate that the URL is the correct one for the supplied Product ID, resulting in
 *       return code TBC_CONNCHECK_MISMATCH (-1) if the validation fails.
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TBC_OK                 (  1) - Normal execution.
 * * TBC_CONNCHECK_MISMATCH ( -1) - Successfully reached the Trackerbird Server, but the URL and ProductID are not consistent.
 * * TBC_CONN_ERROR         ( -2) - Unsuccessful due to comms issue with the Trackerbird Server.
 * * TBC_AUTH_FAILURE       ( -3) - Unsuccessful due to login issue with the Trackerbird Server.
 * * TBC_SERVER_ERROR       ( -4) - Unsuccessful due to internal issue with the Trackerbird Server.
 * * TBC_INTERNAL_ERROR     (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
-(TBCRESULT)checkConnectionToURL:(NSString*)url
                   withProductID:(NSString*)productID;

/**
 * @details This function requests a Manual ReachOut™ Message from the Trackerbird Server. This can be either a plain text
 * Message or a URL Message, depending on what Messages are waiting on the Trackerbird Server.  To request a Manual ReachOut™
 * Message of a specific type, then use function checkMessageForType (@see checkMessageForType).
 *
 * @param message
 * This is an out parameter.  The Trackerbird SDK will always update the mutable string object.  This object should be release'd
 * when the Trackerbird SDK Client no longer needs it.  The value of the string will be one of the following:
 * * If return code >= 0, the value is the Manual ReachOut™ Message.  Max length will be 256 characters plus the NULL.
 * * Otherwise, the value is an empty string.
 *
 * @param messageType
 * This is an out parameter.  The value is set to indicate the ReachOut™ Message Type that was received.  The possible values
 * are as follows:
 * * If return code >= 0, the messageType will be TBC_MESSAGETYPE_TEXT (1) or TBC_MESSAGETYPE_URL (2) to describe Message Type
 *       in message.
 * * Otherwise, the messageType will be undefined.
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * If =0, no message available on the Trackerbird Server.  Memory for message is allocated, and message value is empty string.
 * * If >0, the message count on the Trackerbird Server including the one returned now (so remaining messages are return value
 *       minus 1).  Memory for message is allocated, and message value is the Manual ReachOut™ Message.
 * * If <0, an error has occurred.  Memory for message is allocated, and message value is empty string.
 * * TBC_FUNCTION_NOT_AVAIL    ( -1) - Message Check threshold limit has been reached.
 * * TBC_CONN_ERROR            ( -2) - Unsuccessful due to communications issue with the Trackerbird Server.
 * * TBC_AUTH_FAILURE          ( -3) - Unsuccessful due to login issue with the Trackerbird Server.
 * * TBC_SERVER_ERROR          ( -4) - Unsuccessful due to internal issue with the Trackerbird Server.
 * * TBC_NOT_STARTED           ( -7) - Configuration not successfully started.
 * * TBC_INVALID_PARAMETER     ( -9) - message or messageType are NULL.
 * * TBC_APP_STOPPED           (-14) - Function not available as stopWithSync has already been called.
 * * TBC_INTERNAL_ERROR        (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
-(TBCRESULT)checkMessageReturningMessage:(NSMutableString*)message
                             messageType:(int*)messageType;

/**
 * @details This function requests a Manual ReachOut™ Message from the Trackerbird Server of a specific Message Type (either
 * a plain text Message or a URL Message).  To request a Manual ReachOut™ Message regardless of Message Type, then use function
 * checkMessageReturningMessage (@see checkMessageReturningMessage).
 *
 * @param message
 * This is an out parameter.  The Trackerbird SDK will always update the mutable string object.  This object should be release'd
 * when the Trackerbird SDK Client no longer needs it.  The value of the string will be one of the following:
 * * If return code >= 0, the value is the Manual ReachOut™ message.  Maximum length will be 256 characters plus the NULL.
 * * Otherwise, the value is an empty string.
 *
 * @param messageType
 * The Message Type to request.  The possible values are as follows:
 * * TBC_MESSAGETYPE_ALL  (0) - Request any Message Type (i.e., Text or URL).
 * * TBC_MESSAGETYPE_TEXT (1) - Request only Text Message Type.
 * * TBC_MESSAGETYPE_URL  (2) - Request only URL Message Type.
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * If =0, no message available on the Trackerbird Server.  Memory for message is allocated, and message value is empty string.
 * * If >0, the message count on the Trackerbird Server including the one returned now (so remaining messages are return value
 *       minus 1).  Memory for message is allocated, and message value is the Manual ReachOut™ Message.
 * * If <0, an error has occurred.  Memory for message is allocated, and message value is empty string.
 * * TBC_FUNCTION_NOT_AVAIL    ( -1) - Message Check threshold limit has been reached.
 * * TBC_CONN_ERROR            ( -2) - Unsuccessful due to communications issue with the Trackerbird Server.
 * * TBC_AUTH_FAILURE          ( -3) - Unsuccessful due to login issue with the Trackerbird Server.
 * * TBC_SERVER_ERROR          ( -4) - Unsuccessful due to internal issue with the Trackerbird Server.
 * * TBC_NOT_STARTED           ( -7) - Configuration not successfully started.
 * * TBC_INVALID_PARAMETER     ( -9) - message is NULL.
 * * TBC_APP_STOPPED           (-14) - Function not available as stopWithSync has already been called.
 * * TBC_INTERNAL_ERROR        (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
-(TBCRESULT)checkMessageForType:(int)messageType
               returningMessage:(NSMutableString*)message;

/**
 * @details This function validates a license key with the Blacklist stored on the Trackerbird Server.  Five (5) License Status
 * Flags are returned for the supplied license key.  These values can be set with function setLicenseInformationForKeyType (@see setLicenseInformationForKeyType).
 * Function checkKey is a passive function that only returns data from the Trackerbird Server.  To combine a key check with a
 * key registration, use function changeKey (@see changeKey).
 *
 * @param key
 * The license key to be tested.  The value cannot be NULL or empty.
 *
 * @param licenseArray
 * This is an out parameter.  The Trackerbird SDK will always update the array object.  This object should be release'd
 * when the Trackerbird SDK Client no longer needs it.  The value will be a 5-element integer array containing the License
 * Status Flag values at the following indexes:
 * * TBC_KEY_ACTIVE_INDEX    (0)
 * * TBC_KEY_BLACKLIST_INDEX (1)
 * * TBC_KEY_EXPIRED_INDEX   (2)
 * * TBC_KEY_WHITELIST_INDEX (3)
 * * TBC_KEY_TYPE_INDEX      (4)
 *
 * NOTE:  Any of the array elements can be set to -1 to indicate the data is not available.
 *
 * Each of the first four (4) array elements will be set to one of the following values:
 * * TBC_KEY_STATUS_UNCHANGED (-1) - Meaning Key Status is Unknown.
 * * TBC_KEY_STATUS_NO        ( 0)
 * * TBC_KEY_STATUS_YES       ( 1)
 *
 * The last array element (KEYTYPE) will be set to one of the following License Type values:
 * * TBC_FUNCTION_NOT_AVAILABLE (-1) - Meaning License Management has been disabled, Key Type or Key is unknown, or there was a
 *                                    communications problem with the Trackerbird Server.
 * * TBC_KEYTYPE_EVALUATION     ( 0)
 * * TBC_KEYTYPE_PURCHASED      ( 1)
 * * TBC_KEYTYPE_FREEWARE       ( 2)
 * * TBC_KEYTYPE_UNKNOWN        ( 3)
 * * TBC_KEYTYPE_NFR            ( 4)
 * * TBC_KEYTYPE_CUSTOM1        ( 5)
 * * TBC_KEYTYPE_CUSTOM2        ( 6)
 * * TBC_KEYTYPE_CUSTOM3        ( 7)
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TBC_OK                    (  1) - Normal execution.
 * * TBC_FUNCTION_NOT_AVAIL    ( -1) - Key Check threshold limit has been reached.
 * * TBC_CONN_ERROR            ( -2) - Unsuccessful due to communications issue with the Trackerbird Server.
 * * TBC_AUTH_FAILURE          ( -3) - Unsuccessful due to login issue with the Trackerbird Server.
 * * TBC_SERVER_ERROR          ( -4) - Unsuccessful due to internal issue with the Trackerbird Server.
 * * TBC_NOT_STARTED           ( -7) - Configuration not successfully started.
 * * TBC_INVALID_PARAMETER     ( -9) - licenseArray is NULL, or key is NULL or empty.
 * * TBC_APP_STOPPED           (-14) - Function not available as stopWithSync has already been called.
 * * TBC_INTERNAL_ERROR        (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
-(TBCRESULT)   checkKey:(NSString*)key
  returningLicenseArray:(NSMutableArray*)licenseArray;

/**
 * @details This function validates and registers a new license key with the Trackerbird Server.  Function changeKey is
 * similar to function checkKey (@see checkKey), except rather than just being a passive license check, function
 * changeKey also registers the new key with the Trackerbird Server and associates it with this particular Trackerbird SDK
 * Client.
 *
 * Five (5) License Status Flags are returned for the supplied license key.  These values can be set with function setLicenseInformationForKeyType
 * (@see setLicenseInformationForKeyType).
 *
 * @param newKey
 * The license key to be tested and changed.  The value cannot be NULL or empty.
 *
 * @param licenseArray
 * This is an out parameter.  The Trackerbird SDK will always update the array object.  This object should be release'd
 * when the Trackerbird SDK Client no longer needs it.  The value will be a 5-element integer array containing the License
 * Status Flag values at the following indexes:
 * * TBC_KEY_ACTIVE_INDEX    (0)
 * * TBC_KEY_BLACKLIST_INDEX (1)
 * * TBC_KEY_EXPIRED_INDEX   (2)
 * * TBC_KEY_WHITELIST_INDEX (3)
 * * TBC_KEY_TYPE_INDEX      (4)
 *
 * NOTE:  Any of the array elements can be set to -1 to indicate the data is not available.
 *
 * Each of the first four (4) array elements will be set to one of the following values:
 * * TBC_KEY_STATUS_UNCHANGED (-1) - Meaning Key Status is Unknown.
 * * TBC_KEY_STATUS_NO        ( 0)
 * * TBC_KEY_STATUS_YES       ( 1)
 *
 * The last array element (KEYTYPE) will be set to one of the following License Type values:
 * * TBC_FUNCTION_NOT_AVAILABLE (-1) - Meaning License Management has been disabled, Key Type or Key is unknown, or there was a
 *                                    communications problem with the Trackerbird Server.
 * * TBC_KEYTYPE_EVALUATION     ( 0)
 * * TBC_KEYTYPE_PURCHASED      ( 1)
 * * TBC_KEYTYPE_FREEWARE       ( 2)
 * * TBC_KEYTYPE_UNKNOWN        ( 3)
 * * TBC_KEYTYPE_NFR            ( 4)
 * * TBC_KEYTYPE_CUSTOM1        ( 5)
 * * TBC_KEYTYPE_CUSTOM2        ( 6)
 * * TBC_KEYTYPE_CUSTOM3        ( 7)
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TBC_OK                    (  1) - Normal execution.
 * * TBC_FUNCTION_NOT_AVAIL    ( -1) - Key Check threshold limit has been reached.
 * * TBC_CONN_ERROR            ( -2) - Unsuccessful due to communications issue with the Trackerbird Server.
 * * TBC_AUTH_FAILURE          ( -3) - Unsuccessful due to login issue with the Trackerbird Server.
 * * TBC_SERVER_ERROR          ( -4) - Unsuccessful due to internal issue with the Trackerbird Server.
 * * TBC_NOT_STARTED           ( -7) - Configuration not successfully started.
 * * TBC_INVALID_PARAMETER     ( -9) - licenseArray is NULL, or newKey is NULL or empty.
 * * TBC_APP_STOPPED           (-14) - Function not available as stopWithSync has already been called.
 * * TBC_INTERNAL_ERROR        (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
-(TBCRESULT)  changeKey:(NSString*)newKey
  returningLicenseArray:(NSMutableArray*)licenseArray;

@end
